package com.sun.corba.se.impl.presentation.rmi;

import com.sun.corba.se.spi.presentation.rmi.DynamicMethodMarshaller;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import org.omg.CORBA.portable.UnknownException;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.ResponseHandler;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.ORB;
import org.omg.PortableServer.POAPackage.ServantNotActive;
import org.omg.PortableServer.POAPackage.ObjectNotActive;
import org.omg.PortableServer.POAPackage.WrongPolicy;
import org.omg.CORBA.Object;
import org.omg.PortableServer.POA;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;
import com.sun.corba.se.spi.presentation.rmi.PresentationManager;
import java.rmi.Remote;
import javax.rmi.CORBA.Tie;
import org.omg.PortableServer.Servant;

public final class ReflectiveTie extends Servant implements Tie
{
    private Remote target;
    private PresentationManager pm;
    private PresentationManager.ClassData classData;
    private ORBUtilSystemException wrapper;
    
    public ReflectiveTie(PresentationManager pm, ORBUtilSystemException wrapper) {
        this.target = null;
        this.classData = null;
        this.wrapper = null;
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new DynamicAccessPermission("access"));
        }
        this.pm = pm;
        this.wrapper = wrapper;
    }
    
    public String[] _all_interfaces(POA poa, byte[] array) {
        return this.classData.getTypeIds();
    }
    
    public void setTarget(Remote target) {
        this.target = target;
        if (target == null) {
            this.classData = null;
        }
        else {
            this.classData = this.pm.getClassData(target.getClass());
        }
    }
    
    public Remote getTarget() {
        return this.target;
    }
    
    public Object thisObject() {
        return this._this_object();
    }
    
    public void deactivate() {
        try {
            this._poa().deactivate_object(this._poa().servant_to_id(this));
        }
        catch (WrongPolicy wrongPolicy) {}
        catch (ObjectNotActive objectNotActive) {}
        catch (ServantNotActive servantNotActive) {}
    }
    
    public ORB orb() {
        return this._orb();
    }
    
    public void orb(ORB orb) {
        try {
            com.sun.corba.se.spi.orb.ORB orb2 = (com.sun.corba.se.spi.orb.ORB)orb;
            ((org.omg.CORBA_2_3.ORB)orb).set_delegate(this);
        }
        catch (ClassCastException ex) {
            throw this.wrapper.badOrbForServant(ex);
        }
    }
    
    public OutputStream _invoke(String s, InputStream inputStream, ResponseHandler responseHandler) {
        Method method = null;
        DynamicMethodMarshaller dynamicMethodMarshaller = null;
        try {
            org.omg.CORBA_2_3.portable.InputStream inputStream2 = (org.omg.CORBA_2_3.portable.InputStream)inputStream;
            method = this.classData.getIDLNameTranslator().getMethod(s);
            if (method == null) {
                throw this.wrapper.methodNotFoundInTie(s, this.target.getClass().getName());
            }
            dynamicMethodMarshaller = this.pm.getDynamicMethodMarshaller(method);
            java.lang.Object invoke = method.invoke(this.target, dynamicMethodMarshaller.readArguments(inputStream2));
            org.omg.CORBA_2_3.portable.OutputStream outputStream = (org.omg.CORBA_2_3.portable.OutputStream)responseHandler.createReply();
            dynamicMethodMarshaller.writeResult(outputStream, invoke);
            return outputStream;
        }
        catch (IllegalAccessException ex) {
            throw this.wrapper.invocationErrorInReflectiveTie(ex, method.getName(), method.getDeclaringClass().getName());
        }
        catch (IllegalArgumentException ex2) {
            throw this.wrapper.invocationErrorInReflectiveTie(ex2, method.getName(), method.getDeclaringClass().getName());
        }
        catch (InvocationTargetException ex3) {
            Throwable cause = ex3.getCause();
            if ((cause instanceof SystemException)) {
                throw (SystemException)cause;
            }
            if (cause instanceof Exception && dynamicMethodMarshaller.isDeclaredException(cause)) {
                org.omg.CORBA_2_3.portable.OutputStream outputStream2 = (org.omg.CORBA_2_3.portable.OutputStream)responseHandler.createExceptionReply();
                dynamicMethodMarshaller.writeException(outputStream2, (Exception)cause);
                return outputStream2;
            }
            throw new UnknownException(cause);
        }
    }
}
